// Heatbugs application. Copyright (C) 1996-1999 Santa Fe Institute.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

// The Heatbug Model swarm encapsulates all the objects used in the
// simulated heatbug world itself (but not the user interface objects)

#import "Citizen.h"
#import "NhoodProtestValueGrid.h"
#import <space.h> // Grid2d
#import <objectbase/Swarm.h>
#import "WrappedListGrid2d.h"
#import <simtools.h>

@interface ModelSwarm: Swarm
{
  int numPPL;			
  int visionRadius;
  double evaporationRate;
  double diffuseConstant;
  int worldXSize, worldYSize;
  
  double randomMoveProbability;

  BOOL randomizeCitizenUpdateOrder;
  id modelActions;				  // scheduling data structures
  id modelSchedule;

  id <List> citizenList;				  // list of all the citizens
  id <List> holdingList;
  WrappedListGrid2d * positionGrid;				  // objects representing
  NhoodProtestValueGrid *nhoodProtestValueGrid;		   // the world  
}

- getCitizenList;				  // access methods into the

- (id <Grid2d>)getPositionGrid;			  // model swarm. These methods

- (NhoodProtestValueGrid *)getDiscontentGrid;                           // allow the model swarm to
						  // be observed.
- (BOOL)toggleRandomizedOrder;                    // method to toggle the
                                                 // randomization feature
- createCitizen;

+ createBegin: aZone;				  // extra methods you
- createEnd;					  // provide for Swarms
- buildObjects;
- buildActions;
- chooseSubset;

- activateIn: swarmContext;

@end
