// Heatbugs application. Copyright (C) 1996-1999 Santa Fe Institute.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import "ModelSwarm.h"
#import <random.h>
#import <space.h>
#import "MyArguments.h"
#import <simtools.h>

@implementation ModelSwarm


- getCitizenList
{
  return citizenList;
}

- (WrappedListGrid2d*)getPositionGrid
{
  return positionGrid;
}

- (NhoodProtestValueGrid *)getDiscontentGrid
{
  return nhoodProtestValueGrid;
}


- (BOOL)toggleRandomizedOrder
{
  randomizeCitizenUpdateOrder = 
    ( (randomizeCitizenUpdateOrder == NO) ? YES : NO );
  return (randomizeCitizenUpdateOrder);
}

- createCitizen
{
      Citizen * aCitizen;
      double  personalCondition, threshold;
      
      // Choose a random ideal temperature, output heat from the specified
      // range (model parameters).
      
      personalCondition = [uniformDblRand
                    getDoubleWithMin: .3  withMax: 1];
      threshold = [uniformDblRand
                     getDoubleWithMin: .4 withMax: .7];
      
      
      // Create the heatbug, set the creation time variables
      
      aCitizen = [Citizen createBegin: self];
      [aCitizen  setPositionGrid: positionGrid localProtestValueGrid: nhoodProtestValueGrid];
      aCitizen = [aCitizen createEnd];
      
      // Add the bug to the end of the list.
      // Now initialize the rest of the heatbug's state.
      
      [aCitizen setPersonalCondition: personalCondition];
      [aCitizen setProtestThreshold: threshold];
      [aCitizen setRandomMoveProbability: randomMoveProbability];
      
      [aCitizen setX: [uniformIntRand
                    getIntegerWithMin: 0L
                    withMax: (worldXSize-1)]  // random position
            Y: [uniformIntRand getIntegerWithMin: 0L withMax: (worldYSize-1)]];
   
      [citizenList addLast: aCitizen];
    
  return  aCitizen;
}


+ createBegin: aZone
{
  ModelSwarm *obj;
  id <ProbeMap> probeMap;

  // First, call our superclass createBegin - the return value is the
  // allocated HeatbugModelSwarm object.

  obj = [super createBegin: aZone];

  // Now fill in various simulation parameters with default values.

  
  if( [arguments getNumpplArg]!= -1)
  obj->numPPL=[arguments getNumpplArg];
  else obj->numPPL=100;
//   obj->evaporationRate = 0.99;
//   obj->diffuseConstant = 1.0;
  obj->worldXSize = 80;
  obj->worldYSize = 80;
  obj->visionRadius = 2;
  obj->randomizeCitizenUpdateOrder = NO;
  obj->randomMoveProbability = 0.0;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // Add in a bunch of variables, one per simulation parameter

  [probeMap addProbe: [probeLibrary getProbeForVariable: "numPPL"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "visionRadius"
 				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldXSize"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldYSize"
				    inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "evaporationRate"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForMessage: 
                                      "toggleRandomizedOrder" 
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "randomMoveProbability"
				    inClass: [self class]]];


  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  return obj;
}


- createEnd
{
  return [super createEnd];
}



- buildObjects
{
  int i;

  [super buildObjects];

   positionGrid = [WrappedListGrid2d create: self setSizeX: worldXSize Y: worldYSize];

   nhoodProtestValueGrid = [NhoodProtestValueGrid createBegin: self];
   [nhoodProtestValueGrid setSizeX: worldXSize Y: worldYSize];
    nhoodProtestValueGrid = [nhoodProtestValueGrid createEnd];
    //  [nhoodProtestValueGrid fillWithValue:  0];

   [nhoodProtestValueGrid initializeLattice]; 
   [nhoodProtestValueGrid setPositionGrid: positionGrid];
 
   [nhoodProtestValueGrid setVisionRadius: visionRadius];
   [nhoodProtestValueGrid setNumPPL: numPPL];
   


   holdingList = [List create: self];


   citizenList = [List create: self];
  
   [positionGrid setOverwriteWarnings: 0];

   for (i = 0; i < numPPL; i++)
     {
      printf("citcreate \n");
      [self createCitizen];
    }
  
  [positionGrid setOverwriteWarnings: 1];		  // ok, done cheating.
  
  return self;
}

// Here is where the model schedule is built, the data structures
// that define the simulation of time in the model. The core is an
// actionGroup that has a list of actions. That's then put in a Schedule.

- buildActions
{
  [super buildActions];
  


  modelActions = [ActionGroup create: self]; 
 //   [modelActions createActionTo: self message: M(chooseSubset)];
//    [modelActions createActionForEach: holdingList  message: M(step)];     
    
 
  //  [modelActions createActionTo:   nhoodProtestValueGrid   message: M(eraseProtestGrid)];
//       if (randomizeCitizenUpdateOrder == YES)
// 	{ 
//        [[modelActions createActionForEach: citizenList message: M(updateBeliefs)] 
//           setDefaultOrder: Randomized];
//        [[modelActions createActionForEach: citizenList message: M(chooseLocation)] 
//           setDefaultOrder: Randomized];
//        [[modelActions createActionForEach: citizenList message: M(considerJoiningProtest)] 
//           setDefaultOrder: Randomized];
//         }
//       else
// 	{
//       [modelActions createActionForEach: citizenList message: M(updateBeliefs)] ;
//       [modelActions createActionForEach: citizenList message: M(chooseLocation)] ;
//       [modelActions createActionForEach: citizenList message: M(considerJoiningProtest)];
// 	}
      //COMMENT: citizens can either look at the grid as it was when
      //last updated (option 1) or they can do a survey of their
      //neighborhood as it is currently, taking into account the
      //actions of other agents who have already acted in the same
      //turn (option 2). Above is option 1.  Option 2 is this:

      //Option 2:
      [modelActions createActionForEach: citizenList message: M(step)] ;
 
//     [modelActions createActionTo: nhoodProtestValueGrid message: M(fillWithValue:):(long)0];

   [modelActions createActionTo:   nhoodProtestValueGrid     message: M(stepRule)];     
 
   modelSchedule = [Schedule createBegin: self];
   [modelSchedule setRepeatInterval: 1];
   modelSchedule = [modelSchedule createEnd];
   [modelSchedule at: 0 createAction: modelActions];

  return self;
}

// Now set up the model's activation. swarmContext indicates where
// we're being started in - typically, this model is run as a subswarm
// of an observer swarm.

- chooseSubset
{
  [holdingList removeAll];
  [NSelect select: 30 from: citizenList into: holdingList];
 
  return holdingList;
}

- activateIn: swarmContext
{
  // First, activate ourselves via the superclass activateIn: method.
  // Just pass along the context: the activity library does the right thing.

  [super activateIn: swarmContext];

  // Now activate our own schedule.

  [modelSchedule activateIn: self];

  // Finally, return our activity.

  return [self getActivity];
}

@end
