// Heatbugs application. Copyright (C) 1996-1999 Santa Fe Institute.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

// Bits to support a specialization of diffusion objects: "heat space".
// Most of the real work is done in Diffuse, which implements a CA.
// These functions simplify and stereotype access to the space variable,
// making the Heatbug code higher level.

#import <random.h>
#import "NhoodProtestValueGrid.h"
#import "ListWrapper.h"
#import <math.h>

@implementation Cell

- setX: (int)theX
{
    x = theX;
    return self;
}
- setY: (int)theY
{
    y = theY;
    return self;
}
- (int)getX
{ 
    return x;
}
- (int)getY
{
    return y;
}
@end

@implementation NhoodProtestValueGrid

// Add heat to the current spot. This code checks the bounds on maxHeat,
// pegs value at the top.
+ createBegin: aZone
{
  id r;

  r = [super createBegin: aZone];
  return r;
}  

- createEnd
{
  [super createEnd];
  return self;
}



PHASE(Using)



- initializeLattice
{
  unsigned x, y;

  for (y = 0; y < ysize; y++)
    for (x = 0; x < xsize; x++)
      {
        long newState;

#ifdef RANDOMINIT
        newState = [uniformIntRand getIntegerWithMin: 0L withMax: 2L] == 2;
       
#else
        newState = 0L;
#endif
	[self putValue: newState atX: x Y: y];

      }

  return self;
}



-setPositionGrid: p
{
 positionGrid=p;
 return self;
}

-setNumPPL:(int)n
{
  numPPL=n;
  return self;
}

-setVisionRadius: (int) rad
{
  radius=rad;
  return self ;
}

-(int) getPctProtesting
{
  printf("NHOOD Figures %d %d \n", numProtesters, numPPL);
  return 100.0*numProtesters/numPPL;
}

- stepRule
{
  long newState;
  unsigned xx, yy;
  for (xx = 0; xx < xsize; xx++)
    for (yy = 0; yy < ysize; yy++)
      {
       double pct;
	
        //From each square, look r squares in each direction, get amount of protest activity.       
       pct= [self   measureProtestInVicinityX: xx Y: yy Radius: radius];          

       *(discrete2dSiteAt(lattice, offsets, xx, yy)) = (id)((int)(floor(pct+0.5)));
      }
  return self;
}

-(double) measureProtestInVicinityX: (int) x Y: (int)  y Radius: (int) rad
{
  int r,c;
  double pct;
    int sum=0;
    int sum2=0;
    int xcoord,ycoord;
        for(r=0;r<(2*rad)+1;r++)
	  {
	   xcoord=(x+xsize+r-rad) % xsize;
	   for(c=0;c<(2*rad)+1;c++)
	      {
		int a,b; float spotpct;
		ycoord= (y+ysize+c-radius) % ysize;

		a= [[positionGrid getObjectAtX: xcoord Y: ycoord] getNumProtesters]; 
		b= [positionGrid numberOfObjectsAtX: xcoord Y:ycoord];
		//printf("cell %d %d numprotesters: %d population: %d \n",r,c,a,b);

		sum+= a ;
		sum2 += b;
	      }

	    if(sum2==0) 
	      pct=0;
	    else
	      pct=(100.0*sum/sum2);
	  }
	  
       
	  
return pct;
}

-eraseProtestGrid
{

  // numProtesters=0;
   return self;
}


-addProtester
{
    numProtesters=numProtesters+1;    
    return self;
}

-removeProtester
{
  numProtesters=numProtesters-1;    
  return self;
}



// Search the 9 cell neighbourhood for the requested extreme (cold or hot)
// The X and Y arguments are used both as input (where to search from)
// and as output (pointers are filled with the coordinate of the extreme).
// Note that wraparound edges (boundary conditions) are implicitly in
// the code - look at the call to [self getValueAtX:Y:].

// - (HeatValue) findExtremeType: (HeatExtremeType) type X: (int *) px Y: (int *) py
// {
//   HeatValue bestHeat;
//   int x, y;
//   int bestX, bestY;
//   id <List>heatList;
//   id  cell, bestCell;
//   int offset;
//   char buf[100];

//   // prime loop: assume extreme is right where we're standing
//   bestHeat = [self getValueAtX: *px Y: *py];

//   // Now scan through the world, finding the best cell in the 8 cell
//   // nbd.  To remove the bias from the choice of location, we keep a
//   // list of all best ones and then choose a random location if there
//   // are points of equal best heat.
//   heatList = [List create: [self getZone]];

//   for (y = *py - 1; y <= *py + 1; y++) {  
//       for (x = *px - 1; x <= *px + 1; x++) {
//           HeatValue heatHere;
//           BOOL hereIsBetter, hereIsEqual;

//           heatHere = [self getValueAtX: (x+xsize)%xsize Y: (y+ysize)%ysize];
//           hereIsBetter = (type == cold) ? (heatHere < bestHeat)
//               : (heatHere > bestHeat);
          
//           // printf("@ (%d,%d) = %d ", x, y, heatHere);
          
//           hereIsEqual = (heatHere == bestHeat);
          
//           if (hereIsBetter) {	      // this spot more extreme
              
//               cell = [[[HeatCell create: [self getZone]] setX: x] setY: y];
              
//               // this heat must be the best so far, so delete all the
//               // other cells we have accumulated
//               [heatList deleteAll];
//               [heatList addLast: cell]; // now list only has the one new cell

//               bestHeat = heatHere;   // update information
//           }
          
//           // if we have spots of equal best heat - then we add to the
//           // list from which we can choose randomly later
//           if (hereIsEqual) {
//               cell = [[[HeatCell create: [self getZone]] setX: x] setY: y];
//               [heatList addLast: cell]; // add to the end of the list
//           }
//       }
//       // printf("\n");
//   }
  
//   // printf("---------\n");

//   // choose a random position from the list
//   offset = [uniformIntRand getIntegerWithMin: 0L 
//                            withMax: ([heatList getCount] - 1)];

//   // choose a point at random from the heat list
//   bestCell = [heatList atOffset: offset];

//   // Now we've found the requested extreme. Arrange to return the
//   // information (normalize coordinates), and return the heat we found.

//   *px = ([bestCell getX] + xsize) % xsize;
//   *py = ([bestCell getY] + ysize) % ysize;

//   // clean up the temporary list of (x,y) points
//   [heatList deleteAll];
//   [heatList drop];

//   return ([self getValueAtX: *px Y: *py]);
// }

@end

